package com.displaynone.acss.ui.profile

import android.os.Bundle
import android.util.Log
import android.view.View
import android.widget.Toast
import androidx.activity.OnBackPressedCallback
import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import androidx.navigation.fragment.findNavController
import androidx.recyclerview.widget.LinearLayoutManager
import com.bumptech.glide.Glide
import com.displaynone.acss.R
import com.displaynone.acss.components.auth.models.user.UserServiceST
import com.displaynone.acss.components.auth.models.user.repository.VisitAdapter
import com.displaynone.acss.components.auth.models.user.repository.dto.UserDTO
import com.displaynone.acss.databinding.FragmentProfileBinding
import com.displaynone.acss.ui.profile.ProfileViewModel.Action
import com.displaynone.acss.ui.scan.QrScanDestination
import com.displaynone.acss.util.collectWithLifecycle
import com.displaynone.acss.util.navigateTo

class ProfileFragment: Fragment(R.layout.fragment_profile) {
    private var _binding: FragmentProfileBinding? = null
    private val binding: FragmentProfileBinding get() = _binding!!

    private val viewModel: ProfileViewModel by viewModels()
    private val adapter = VisitAdapter()

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        _binding = FragmentProfileBinding.bind(view)
        checkForAdmin()

        binding.swipeRefresh.setOnRefreshListener {
            if (getIsMe()){
                refreshData()
            } else{
                showData(getUserDto()!!)
            }
        }
        binding.logout.setOnClickListener{
            logout()
        }
        binding.scan.setOnClickListener{
            navigateTo(view, R.id.action_profileFragment_to_qrScanFragment)
        }
        binding.buttonSearch.setOnClickListener{
            navigateTo(view, R.id.action_profileFragment_to_adminFragment)
        }
        binding.recyclerViewLogs.adapter = adapter
        if (getIsMe()) {
            refreshData()
            viewModel.visitListState.collectWithLifecycle(this) { data ->
                adapter.submitData(data)
            }
            waitForQRScanResult()
        } else{
            showData(getUserDto()!!)
            Log.d("ProfileFragment", "set login")
            viewModel.visitListStateFromLogin.collectWithLifecycle(this) { data ->
                adapter.submitData(data)
                Log.d("ProfileFragment", "adapter submitted data")
            }
            hideButtons()
        }
        subscribe()
        binding.recyclerViewLogs.layoutManager = LinearLayoutManager(requireContext())
//        viewModel.visitListStateFromLogin.collectWithLifecycle(this) { data ->
//            adapter.submitData(data)
//        }

    }
    private fun checkForAdmin() {
        Log.d("check", "cheking for roles")

        val userDTO = UserServiceST.getInstance().getUserDTO()
        if (userDTO != null) {
            if (userDTO.roles.any { it.name == "ROLE_ADMIN" }) {
                Log.d("adminlog", "i'm admin")
                binding.buttonSearch.visibility = View.VISIBLE
                binding.rightsUsingSmartphone.text = "Пропуск действителен"
            }
            if (userDTO.roles.any { it.name == "ROLE_USER" }) {
                Log.d("userlog", "i'm user")

                binding.rightsUsingSmartphone.text = "Пропуск действителен"
            }
        }
    }
    private fun hideButtons() {
        binding.logout.visibility = View.GONE
        binding.scan.visibility = View.GONE
        binding.buttonSearch.visibility = View.GONE
    }
    fun showMyData(userDTO: UserDTO){
        binding.fio.text = userDTO.name
        binding.position.text = userDTO.position
        setAvatar(userDTO.photo)
    }
    fun showData(userDTO: UserDTO){
        binding.fio.text = userDTO.name
        binding.position.text = userDTO.position
        viewModel.setLogin(login1 = userDTO.login)
        Log.d("ProfileFragment", userDTO.login)

//        binding.lastEntry.text = userDTO.lastVisit

        setAvatar(userDTO.photo)
    }
    private fun refreshData() {
        Log.d("ProfileFragment", "Refreshed")
        viewModel.getInfo()
        subscribeToGetData()
    }
    fun subscribe() {
        viewModel.action.collectWithLifecycle(this) { action ->
            if (action is Action.GoToAuth) {
                view?.let { navigateTo(it, R.id.action_profileFragment_to_authFragment) } ?: throw IllegalStateException("View is null")
            }
            if (action is Action.GoToScan) {
                view?.let { navigateTo(it, R.id.action_profileFragment_to_qrResultFragment) }?: throw IllegalStateException("View is null")
            }
        }
    }
    private fun getUserDto(): UserDTO? {
        return arguments?.getSerializable("user") as? UserDTO
    }
    private fun getIsMe(): Boolean {
        return arguments?.getBoolean("isMe", true) ?: true
    }

    private fun waitForQRScanResult() {

        requireActivity().onBackPressedDispatcher.addCallback(
            viewLifecycleOwner,
            object : OnBackPressedCallback(true) {
                override fun handleOnBackPressed() {

                }
            })

        findNavController().currentBackStackEntry?.savedStateHandle?.getLiveData<Bundle>(
            QrScanDestination.REQUEST_KEY
        )?.observe(viewLifecycleOwner) { bundle ->
            val qrCode = bundle.getString("key_qr")
            if (!qrCode.isNullOrEmpty()) view?.let {
                val bundle = Bundle().apply {
                    putString("qrCode", qrCode)
                }
                navigateTo(it, R.id.action_profileFragment_to_qrResultFragment, bundle)
            }
        }
    }
    private fun logout() {
        viewModel.logout()
        viewModel.openAuth()
        Toast.makeText(activity, "LOGOUT", Toast.LENGTH_SHORT).show()
    }

    private fun subscribeToGetData(){
        viewModel.state.collectWithLifecycle(this) { state ->
            if (state is ProfileViewModel.State.Show) {
                val userDto: UserDTO = state.item
                showMyData(userDto)
            }
        }
    }

    private fun setAvatar(photo: String) {
        Glide.with(requireContext())
            .load(photo)
            .placeholder(R.drawable.ic_photo)
            .error(R.drawable.ic_back)
            .into(binding.avatar)
    }

    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }
}